use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007005
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
API
Alders
Anirvan
ApacheLog
Attrill
Base
Bell
Bertrand
Bucheli
CPAN
Carly
Carsten
Chatterjee
Cholet
Code
Conflicts
DROLSKY
DROLSKY's
Dave
Dispatch
Doug
Dumont
Email
Etheridge
Fandiño
File
Goess
Graham
Gregory
Grohmann
Handle
Jackson
Joelle
Johann
Jonathan
Karen
Kerin
Kivanc
Knop
Konrad
Leschenko
Locked
Log
MIMELite
MailSend
MailSender
MailSendmail
Manfredi
Maslak
Mengué
Michael
Millar
Miyagawa
NTLM
Null
Olaf
Olivier
Ollis
Oschwald
Output
PayPal
Pfeiffer
Rezic
Rohan
Rolschewski
Rolsky
Rolsky's
Ross
STDERR
STDOUT
Salvador
Schilli
Schout
Screen
Sergey
Slaven
Steve
Straup
Subclasses
Swartz
Syslog
TLS
Tatsuhiko
Types
UTF
Vars
Whitney
Yazan
anirvan
apache
appenders
autarch
auth
authid
authpriv
authpwd
autoflushed
classname
crit
dolmen
doug
drolsky
emerg
ether
fd
filename
goschwald
haarg
hartzell
ident
jmaslak
jorol
kb
kern
kfm
kivancyazan
lib
logopt
mail
mschout
multi
olaf
params
plicease
replyto
ross
se456
sergle
sfandino
smtp
srezic
stderr
steveb
subclass's
subclasses
swartz
uucp
whitney
